#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Symbols;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Symbols {
  [StorableClass]
  [Item("Read Memory", "Represents a read memory operation.")]
  public sealed class ReadMem : Symbol {
    #region Propeties
    [Storable]
    private int minAddress;
    public int MinAddress {
      get { return minAddress; }
      set {
        if (value != minAddress) {
          minAddress = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }
    [Storable]
    private int maxAddress;
    public int MaxAddress {
      get { return maxAddress; }
      set {
        if (value != maxAddress) {
          maxAddress = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }
    #endregion
    [StorableConstructor]
    private ReadMem(bool deserializing) : base(deserializing) { }
    private ReadMem(ReadMem original, Cloner cloner)
      : base(original, cloner) {
      minAddress = original.minAddress;
      maxAddress = original.maxAddress;
    }
    public ReadMem()
      : base("ReadMem", "Represents a read memory operation.") {
      minAddress = 0;
      maxAddress = 3;
    }

    public override SymbolicExpressionTreeNode CreateTreeNode() {
      return new ReadMemTreeNode(this);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ReadMem(this, cloner);
    }
  }
}
